@@ -6,13 +6,14 @@ This is a work in progress, but ultimately the goal is to be able to express
 concise typing information in docstrings and then explicitly expose that to
 Python.
 
-
 Requirements:
     pip install mypy autoflake yapf
 
 CommandLine:
     # Run script to parse google-style docstrings and write pyi files
-    python ~/code/ubelt/dev/gen_typed_stubs.py
+
+    xdev doctypes --module=xdev
+    python ~/code/xdev/xdev/cli/docstr_stubgen.py
 
     See:
     ~/code/mypy/mypy/stubgen.py
@@ -44,6 +45,7 @@ try:
         # TupleExpr, ListExpr, ComparisonExpr, CallExpr, IndexExpr, EllipsisExpr,
         # ClassDef, MypyFile, Decorator, AssignmentStmt, TypeInfo,
         # IfStmt, ImportAll, ImportFrom, Import,
+        IS_ABSTRACT,
         FuncDef,
         # FuncBase, Block,
         # Statement, OverloadedFuncDef, ARG_POS,
@@ -78,11 +80,19 @@ from typing import (List, Dict, Optional)
 import ubelt as ub
 
 
+Stub = ...  # hack for mypy. Not sure why it is generated in the first place.
+
+
 def _hack_away_compiled_mypy():
+    """
+    Worked with: mypy-0.970+dev.ddbea6988c0913c70ed16cd2fda6064e301b4b63
+    """
     # This doesn't seem to work. The only thing that has worked so far is a
     # custom checkout and developer install. Not sure why that is the case.
     modpath = ub.Path(ub.modname_to_modpath('mypy'))
+    print(f'modpath={modpath}')
     compiled_modules = list(modpath.glob('*.so'))
+    print(f'compiled_modules={compiled_modules}')
     for p in compiled_modules:
         p.delete()
 
@@ -252,11 +262,12 @@ def generate_typed_stubs(modpath):
         files.append(target)
         with stubgen.generate_guarded(mod.module, target, options.ignore_errors, options.verbose):
             stubgen.generate_stub_from_ast(mod, target, options.parse_only,
-                                           options.pyversion,
+                                           # options.pyversion,
                                            options.include_private,
                                            options.export_less)
 
-            gen = ExtendedStubGenerator(mod.runtime_all, pyversion=options.pyversion,
+            gen = ExtendedStubGenerator(mod.runtime_all,
+                                        # pyversion=options.pyversion,
                                         include_private=options.include_private,
                                         analyzed=not options.parse_only,
                                         export_less=options.export_less)
@@ -273,13 +284,13 @@ def generate_typed_stubs(modpath):
                 # Hack for kwcoco
                 'ObjT',
             }
-            for type_var_name in set(gen.import_tracker.required_names) & set(known_one_letter_types):
+            for type_var_name in sorted(set(gen.import_tracker.required_names) & set(known_one_letter_types)):
                 gen.add_typing_import('TypeVar')
                 # gen.add_import_line('from typing import {}\n'.format('TypeVar'))
                 gen._output = ['{} = TypeVar("{}")\n'.format(type_var_name, type_var_name)] + gen._output
 
             custom_types = {'Hasher', 'Sliceable'}
-            for type_var_name in set(gen.import_tracker.required_names) & set(custom_types):
+            for type_var_name in sorted(set(gen.import_tracker.required_names) & set(custom_types)):
                 gen.add_typing_import('TypeVar')
                 # gen.add_import_line('from typing import {}\n'.format('TypeVar'))
                 gen._output = ['{} = TypeVar("{}")\n'.format(type_var_name, type_var_name)] + gen._output
@@ -320,11 +331,26 @@ def postprocess_hacks(text, mod):
     # text = text.replace('odict = OrderedDict', '')
     # text = text.replace('ddict = defaultdict', '')
 
+    # FIXME: does ubelt still need this?
+    if mod.path.endswith('util_dict.py'):
+        # hack for util_dict
+        text = 'import sys\n' + text
+
     if mod.path.endswith('util_path.py'):
         # hack for forward reference
         text = text.replace(' -> Path:', " -> 'Path':")
         text = text.replace('class Path(_PathBase)', "class Path")
 
+    # Not sure why this happens
+    text = text.replace('from io import io\n', '')
+    text = text.replace('from datetime import datetime\n', '')
+
+    # Ubelt hack
+    if 'DictBase' in text:
+        # Hack for util_dict
+        text = text.replace('DictBase = OrderedDict\n', '')
+        text = text.replace('DictBase = dict\n', 'DictBase = OrderedDict if sys.version_info[0:2] <= (3, 6) else dict')
+
     # Format the PYI file nicely
     text = autoflake.fix_code(text, remove_unused_variables=True,
                               remove_all_unused_imports=True)
@@ -474,6 +500,8 @@ def common_unreferenced():
         pass
 
     unref = [
+        {'name': 'datetime', 'modname': 'datetime'},
+        {'name': 'io', 'modname': 'io'},
         {'name': 'PathLike', 'modname': 'os'},
         {'name': 'ModuleType', 'modname': 'types'},
         {'name': 'FrameType', 'modname': 'types'},
@@ -516,6 +544,27 @@ def hacked_typing_info(type_name):
             add_typing_import(typing_arg)
             add_import_line('from typing import {}\n'.format(typing_arg))
 
+    if 'Float32' in type_name:
+        add_import_line('from nptyping import {}\n'.format('Float32'))
+
+    if 'Int64' in type_name:
+        add_import_line('from nptyping import {}\n'.format('Int64'))
+
+    if 'Shape' in type_name:
+        add_import_line('from nptyping import {}\n'.format('Shape'))
+
+    if 'UInt8' in type_name:
+        add_import_line('from nptyping import {}\n'.format('UInt8'))
+
+    if 'Bool' in type_name:
+        add_import_line('from nptyping import {}\n'.format('Bool'))
+
+    if 'Integer' in type_name:
+        add_import_line('from nptyping import {}\n'.format('Integer'))
+
+    if 'skimage.transform.AffineTransform' == type_name:
+        add_import_line('import skimage.transform\n')
+
     common_modnames = common_module_names()
     common_aliases = common_module_aliases()
 
@@ -703,10 +752,18 @@ class ExtendedStubGenerator(StubGenerator):
                 # The class attributes should override the init signature
                 if init in self._docstring_class_attr_infos:
                     typename = self._docstring_class_attr_infos[init]['type']
-                    annotation = fastparse.parse_type_string(typename, 'Any', 0, 0)
+                    try:
+                        annotation = fastparse.parse_type_string(typename, 'Any', 0, 0)
+                    except Exception:
+                        print(f'FAILED ON typename={typename} for {init}')
+                        annotation = None
                 elif init in name_to_parsed_docstr_info:
                     typename = name_to_parsed_docstr_info[init]['type']
-                    annotation = fastparse.parse_type_string(typename, 'Any', 0, 0)
+                    try:
+                        annotation = fastparse.parse_type_string(typename, 'Any', 0, 0)
+                    except Exception:
+                        print(f'FAILED ON typename={typename} for {init}')
+                        annotation = None
                     # import xdev
                     # xdev.embed()
                 init_code = self.get_init(init, value, annotation=annotation)
@@ -816,7 +873,7 @@ class ExtendedStubGenerator(StubGenerator):
                 retname = None  # implicit Any
             else:
                 retname = self.print_annotation(o.unanalyzed_type.ret_type)
-        elif isinstance(o, FuncDef) and (o.is_abstract or o.name in METHODS_WITH_RETURN_VALUE):
+        elif o.abstract_status == IS_ABSTRACT or o.name in METHODS_WITH_RETURN_VALUE:
             # Always assume abstract methods return Any unless explicitly annotated. Also
             # some dunder methods should not have a None return type.
             retname = None  # implicit Any
@@ -875,6 +932,14 @@ class ExtendedStubGenerator(StubGenerator):
         # We will need to use it in the init method parsing
         self._docstring_class_attr_infos = {}
         parent_mod = ub.import_module_from_name(self.module)
+
+        # Classes we will not make stubs for. TODO: generalize.
+        blocklist = {
+            '_RationalNDArray',
+        }
+        if o.name in blocklist:
+            return
+
         real_class = getattr(parent_mod, o.name, None)
         if real_class is not None and real_class.__doc__ is not None:
             from xdoctest.docstr import docscrape_google
